Option Explicit On
Option Strict On

Imports System.Drawing

''' <summary>
''' Allows for manipulating colors (luminance, saturation, etc) and converting 
''' from RGB to HLS and vice versa.
''' </summary>
''' <remarks>Source: http://www.vbaccelerator.com/home/net/Code/Libraries/Graphics/HLS_to_RGB/article.asp</remarks>
Public Class cHLS_RGB

#Region " --- Members --- "

   Private _bytRed As Byte = 0
   Private _bytGreen As Byte = 0
   Private _bytBlue As Byte = 0

   Private _dblHue As Double = 0
   Private _dblLuminance As Double = 0
   Private _dblSaturation As Double = 0

#End Region

#Region " --- Properties --- "

   Public Property Red() As Byte
      Get
         Return _bytRed
      End Get
      Set(ByVal Value As Byte)
         _bytRed = Value
         ToHLS()
      End Set
   End Property

   Public Property Green() As Byte
      Get
         Return _bytGreen
      End Get
      Set(ByVal Value As Byte)
         _bytGreen = Value
         ToHLS()
      End Set
   End Property

   Public Property Blue() As Byte
      Get
         Return _bytBlue
      End Get
      Set(ByVal Value As Byte)
         _bytBlue = Value
         ToHLS()
      End Set
   End Property

   Public Property Luminance() As Double
      Get
         Return _dblLuminance
      End Get
      Set(ByVal Value As Double)
         If ((Value < 0.0F) Or (Value > 1.0F)) Then
            Throw New ArgumentOutOfRangeException("Luminance", "Luminance must be between 0.0 and 1.0")
         End If
         _dblLuminance = Value
         ToRGB()
      End Set
   End Property

   Public Property Hue() As Double
      Get
         Return _dblHue
      End Get
      Set(ByVal Value As Double)
         If ((Value < 0.0F) Or (Value > 360.0F)) Then
            Throw New ArgumentOutOfRangeException("Hue", "Hue must be between 0.0 and 360.0")
         End If
         _dblHue = Value
         ToRGB()
      End Set
   End Property

   Public Property Saturation() As Double
      Get
         Return _dblSaturation
      End Get
      Set(ByVal Value As Double)
         If ((Value < 0.0F) Or (Value > 1.0F)) Then
            Throw New ArgumentOutOfRangeException("Saturation", "Saturation must be between 0.0 and 1.0")
         End If
         _dblSaturation = Value
         ToRGB()
      End Set
   End Property

   Public Property Color() As Color
      Get
         Dim c As Color = Color.FromArgb(_bytRed, _bytGreen, _bytBlue)
         Return c
      End Get
      Set(ByVal Value As Color)
         _bytRed = Value.R
         _bytGreen = Value.G
         _bytBlue = Value.B
         ToHLS()
      End Set
   End Property

#End Region

#Region " --- Methods --- "

   Public Sub LightenColor(ByVal lightenBy As Double)
      _dblLuminance *= (1.0F + lightenBy)
      If (_dblLuminance > 1.0F) Then
         _dblLuminance = 1.0F
      End If
      ToRGB()
   End Sub

   Public Sub DarkenColor(ByVal darkenBy As Double)
      _dblLuminance *= darkenBy
      ToRGB()
   End Sub

   Private Sub ToHLS()
      Dim minval As Byte = Math.Min(_bytRed, Math.Min(_bytGreen, _bytBlue))
      Dim maxval As Byte = Math.Max(_bytRed, Math.Max(_bytGreen, _bytBlue))

      Dim mdiff As Double = (maxval * 1.0 - minval * 1.0)
      Dim msum As Double = (maxval * 1.0 + minval * 1.0)

      _dblLuminance = msum / 510.0F

      If (maxval = minval) Then
         _dblSaturation = 0.0F
         _dblHue = 0.0F
      Else
         Dim rnorm As Double = (maxval - _bytRed) / mdiff
         Dim gnorm As Double = (maxval - _bytGreen) / mdiff
         Dim bnorm As Double = (maxval - _bytBlue) / mdiff

         If (_dblLuminance <= 0.5F) Then
            _dblSaturation = (mdiff / msum)
         Else
            _dblSaturation = (mdiff / (510.0F - msum))
         End If

         If (_bytRed = maxval) Then
            _dblHue = 60.0F * (6.0F + bnorm - gnorm)
         End If
         If (_bytGreen = maxval) Then
            _dblHue = 60.0F * (2.0F + rnorm - bnorm)
         End If
         If (_bytBlue = maxval) Then
            _dblHue = 60.0F * (4.0F + gnorm - rnorm)
         End If
         If (_dblHue > 360.0F) Then
            _dblHue = _dblHue - 360.0F
         End If
      End If
   End Sub

   Private Sub ToRGB()
      If (_dblSaturation = 0.0) Then
         _bytRed = CByte(_dblLuminance * 255.0F)
         _bytGreen = _bytRed
         _bytBlue = _bytRed
      Else
         Dim rm1 As Double
         Dim rm2 As Double

         If (_dblLuminance <= 0.5F) Then
            rm2 = _dblLuminance + _dblLuminance * _dblSaturation
         Else
            rm2 = _dblLuminance + _dblSaturation - _dblLuminance * _dblSaturation
         End If
         rm1 = 2.0F * _dblLuminance - rm2
         _bytRed = ToRGB1(rm1, rm2, _dblHue + 120.0F)
         _bytGreen = ToRGB1(rm1, rm2, _dblHue)
         _bytBlue = ToRGB1(rm1, rm2, _dblHue - 120.0F)
      End If
   End Sub

   Private Function ToRGB1(ByVal rm1 As Double, ByVal rm2 As Double, ByVal rh As Double) As Byte
      If (rh > 360.0F) Then
         rh -= 360.0F
      ElseIf (rh < 0.0F) Then
         rh += 360.0F
      End If

      If (rh < 60.0F) Then
         rm1 = rm1 + (rm2 - rm1) * rh / 60.0F
      ElseIf (rh < 180.0F) Then
         rm1 = rm2
      ElseIf (rh < 240.0F) Then
         rm1 = rm1 + (rm2 - rm1) * (240.0F - rh) / 60.0F
      End If

      ToRGB1 = CByte(rm1 * 255)
   End Function

#End Region

#Region " --- Constructors --- "

   Public Sub New(ByVal c As Color)
      _bytRed = c.R
      _bytGreen = c.G
      _bytBlue = c.B
      ToHLS()
   End Sub

   Public Sub New(ByVal hue As Double, ByVal luminance As Double, ByVal saturation As Double)
      If ((saturation < 0.0F) Or (saturation > 1.0F)) Then
         Throw New ArgumentOutOfRangeException("Saturation", "Saturation must be between 0.0 and 1.0")
      End If
      If ((hue < 0.0F) Or (hue > 360.0F)) Then
         Throw New ArgumentOutOfRangeException("Hue", "Hue must be between 0.0 and 360.0")
      End If
      If ((luminance < 0.0F) Or (luminance > 1.0F)) Then
         Throw New ArgumentOutOfRangeException("Luminance", "Luminance must be between 0.0 and 1.0")
      End If
      _dblHue = hue
      _dblLuminance = luminance
      _dblSaturation = saturation
      ToRGB()
   End Sub

   Public Sub New(ByVal red As Byte, ByVal green As Byte, ByVal blue As Byte)
      _bytRed = red
      _bytGreen = green
      _bytBlue = blue
   End Sub

   Public Sub New(ByVal hlsrgb As cHLS_RGB)
      _bytRed = hlsrgb.Red
      _bytBlue = hlsrgb.Blue
      _bytGreen = hlsrgb.Green
      _dblLuminance = hlsrgb.Luminance
      _dblHue = hlsrgb.Hue
      _dblSaturation = hlsrgb.Saturation
   End Sub

   Public Sub New()

   End Sub

#End Region

End Class